"""
Get Task Results
"""


def get_task_results(
        **kwargs):
    """get_task_results

    return None for allowing tests to override the returned payloads

    :param **kwargs: keyword arguments
    """

    return None
# end of get_task_results
